/*** 
 * \file gre/3dftter.h
 * \brief GRE::FLATTRIANGULATION defns
 *
 * \if NODOC
 * $Id: 3dftter.h_v 1.11 2011/10/06 15:20:29 mju Exp $
 *
 * $Log: 3dftter.h_v $
 * Revision 1.11  2011/10/06 15:20:29  mju
 * Remove dependencies.
 *
 * Revision 1.10  2005/10/27 13:06:13  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.2  2004/06/17 17:23:55  vdronov
 * added case for non raster flat surface
 *
 * Revision 1.1  2004/06/15 16:06:18  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DFTTER_H
#define	INC_GRE_3DFTTER_H

#ifndef	INC_GRE_3DTERRA_H
#include <gre/3dterra.h>
#endif

#ifndef	INC_MI32_TRANS2D_MAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif


namespace GRE {

//! Flat triangulation terrain model.
class FLATTRIANGULATION : public TERRAIN {
	public:

		FLATTRIANGULATION (											//! Constructor 
			);

		~FLATTRIANGULATION (											//! Destructor 
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DRECT2D ALIGN16(m_ObjExtents);
		DRECT3D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		TRANS2D_MAPGEN m_Transformation;
		bool m_IsRaster;
		TRIANGULATION m_Triangulation;
		BITSET m_TrianglesBits;	
		SCENE3D *m_Scene;

		FLATTRIANGULATION& operator= (								//! Assignment operator (not implemented)
			const FLATTRIANGULATION& rhs
			);

		TERRAIN* v_CreateInstance (
			) { return new FLATTRIANGULATION(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		bool v_IsObjectTypeSupported (
			const RVC::OBJTYPE objtype
			) const { return (objtype == RVC::OBJTYPE_Raster); };

		void v_Free (
			);

		ERRVALUE Create (					 
			const GRE::LAYER *layer
			);

		void Delete (					 
			);

		ERRVALUE v_GetDefaultScene ( 
			SCENE3D& scene
			) const;

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<TEXTURE*> &textures	  
			);

		ERRVALUE v_DrawScene (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			TEXTURE* texture,
			TEXTUREFILTER* texturefilter
			);

		ERRVALUE v_SetScene (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<GEOMETRIC3D*> &geometrics,
			const bool wireframe
			);

		float v_GetElevation (
			const float x,
			const float y
			) const;

		ERRVALUE v_ComputeProfile (
			const POLYLINE& polyline,
			POLYLINE& profile,
			SIMPLE_ARRAY<INT32>& index
			);

		float v_GetAngleToNorth (
			const float x,
			const float y
			) const;

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) const { return m_CoordRefSys; };

		const DRECT3D& v_GetExtents (
			) { return m_Extents; };

		void v_CreateBoundary (
			REGION2D& boundary
			);

		bool v_HasNull (
			) const { return false; };

		bool v_IsNull (
			const float x,
			const float y
			) const;

		ERRVALUE CreateRegularTIN (
			const TRANS2D_MAPGEN& transformation
			);

		void DeleteRegularTIN (
			);

		ERRVALUE SetSceneForRegularTIN (
			const SCENE3D& scene,
			const SIMPLE_ARRAY<TEXTURE*> &textures	  
			);

		ERRVALUE DrawSceneForRegularTIN (
			CONTROLLER &controller,			 
			DRAWINGCONTEXT3D& drawing,
			TEXTURE* texture,
			TEXTUREFILTER* texturefilter
			);

		void TransformRegularTIN (
			const TRANS2D_MAPGEN& transformation
			);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

};		// End of namespace

#endif
