/**
 * \file gre/layerdc.h
 * \brief GRE_LAYERDC class definition
 *
 * \if NODOC
 * $Id: layerdc.h_v 1.82 2012/12/27 22:41:05 ldyas Exp $
 *
 * $Log: layerdc.h_v $
 * Revision 1.82  2012/12/27 22:41:05  ldyas
 * Add DoBorder param to Fill Sub Region Device 2D
 *
 * Revision 1.81  2012/05/02 17:27:50  mju
 * Add getLayerView.
 *
 * Revision 1.80  2012/04/04 20:48:37  mju
 * Adjust methods for stereo shift.
 *
 * Revision 1.79  2012/03/30 17:28:06  mju
 * Support single/dual stereo shifting.
 *
 * Revision 1.78  2011/12/08 20:41:24  mju
 * Use stereoview hdr.
 *
 * Revision 1.77  2011/12/05 21:59:54  mju
 * Remove unused members.
 *
 * Revision 1.76  2011/12/02 18:24:53  mju
 * Add zview member and remove unneeded methods.
 *
 * Revision 1.75  2011/11/22 22:37:38  mju
 * Keep last pattern of each type, not by element type.
 *
 * Revision 1.74  2011/11/21 16:42:09  mju
 * Make drawPointDevice2D public.
 * Add ImageInsertFilterStereo2D.
 *
 * Revision 1.73  2011/11/11 22:11:33  mju
 * Add drawPointObject and add optional stylepattern parm to setStylePoint.
 *
 * Revision 1.72  2011/11/09 22:14:44  mju
 * Add drawSegmentDevice2D.
 *
 * Revision 1.71  2011/11/09 21:01:57  mju
 * Add more drawRegion/SubRegion methods.
 * Pass pointer to style object instead of reference so can be optional.
 *
 * Revision 1.70  2011/11/09 17:36:56  mju
 * Add setupStereo and make setupStereoIn2D private.
 *
 * Revision 1.69  2011/11/08 13:38:05  mju
 * Add more unified drawing methods and rename for clarity.
 *
 * Revision 1.68  2011/11/04 21:43:36  mju
 * Change read...Pattern to return pointer to pattern if needed.
 * Add members for keeping track of last patterns read.
 *
 * Revision 1.67  2011/11/04 17:47:39  mju
 * Add drawSubRegion2DNoTrans and setStylePoint.
 *
 * Revision 1.66  2011/11/03 21:26:29  mju
 * Change drawPoint to use drawflags and add drawPoints.
 *
 * Revision 1.65  2011/11/03 17:43:12  mju
 * Add setStyle..., checkColorHighlight, drawing methods for common geometric layer use.
 *
 * Revision 1.64  2011/11/02 20:38:58  mju
 * Add more draw/fill/calc methods for stereo.
 *
 * Revision 1.63  2011/10/31 20:52:25  mju
 * More stereo.
 *
 * Revision 1.62  2011/10/28 20:09:39  mju
 * Support dual frame buffers for stereo.
 *
 * Revision 1.61  2011/10/20 12:45:48  mju
 * Remove redundant hdrs.
 *
 * Revision 1.60  2011/10/19 17:08:50  mju
 * Change calcSurfaceZ to return value in point.z and add readSurfaceZ.
 *
 * Revision 1.59  2011/10/14 17:49:59  mju
 * Add CalcSurfaceZ and TransToDevice.
 *
 * Revision 1.58  2011/10/13 17:37:23  mju
 * More stereo methods.
 *
 * Revision 1.57  2011/10/10 22:32:37  mju
 * Add members for drawPoint.
 *
 * Revision 1.56  2011/10/06 15:22:22  mju
 * Use scene3d hdr.
 *
 * Revision 1.55  2011/10/03 23:01:20  mju
 * More strero stuff.
 *
 * Revision 1.54  2011/09/30 21:16:36  mju
 * Add stuff for two-pass stereo drawing.
 *
 * Revision 1.53  2011/09/29 12:26:14  mju
 * Use stereodevice class.
 *
 * Revision 1.52  2011/02/07 23:18:45  bcolombe
 * Add optional SectionRect and ForceAlphaBlend to ImageDraw2D
 *
 * Revision 1.50  2010/07/15 21:31:48  mju
 * Add imageDraw2d/InsertResample/ZoomFilter.
 *
 * Revision 1.49  2010/04/23 16:54:19  mju
 * Use unsignedset.
 *
 * Revision 1.48  2008/10/15 12:53:16  mju
 * Use sml/class hdr instead of mi32/sml hdr.
 *
 * Revision 1.44  2007/09/04 13:37:37  mju
 * Add drawRegionClipped2D.
 *
 * Revision 1.43  2007/04/30 19:57:46  mju
 * Stereo stuff.
 *
 * Revision 1.42  2007/04/27 17:47:25  mju
 * Add access methods for stereo.
 *
 * Revision 1.41  2007/03/28 22:02:35  mju
 * Add densifyOnSurface parm to drawLineClipped3D.
 *
 * Revision 1.39  2006/09/18 16:17:08  mju
 * Add drawWireframeFlat.
 *
 * Revision 1.36  2006/09/06 16:43:29  mju
 * Add getSurfaceZ method for single point with search.
 *
 * Revision 1.31  2005/05/03 15:02:25  mju
 * Keep track of render target.
 * Add checkColorLock using elemtype/num and highlightset.
 *
 * Revision 1.30  2005/05/02 20:40:08  mju
 * Add checkColorOverride.
 *
 * Revision 1.29  2005/05/02 17:44:06  mju
 * Add checkColorLock.
 *
 * Revision 1.28  2005/03/01 14:21:15  mju
 * Add methods to access drawn sets.
 *
 * Revision 1.25  2005/02/25 17:47:27  mju
 * Remove setup2D/3D, add CanDrawLayer, Set3dStereoView.
 *
 * Revision 1.19  2005/02/04 18:57:35  mju
 * Add ctors using mgd device.
 * Add setDrawingDevice.
 * Deprecate ctors using mgd context.
 *
 * Revision 1.16  2005/01/24 21:31:36  mju
 * Move members which can be saved between drawing to layerview.
 *
 * Revision 1.9  2003/10/06 20:47:23  mju
 * Add NotifyDrawElementBegin/End.
 *
 * Revision 1.7  2003/09/10 17:03:44  scowan
 * Create no trans draw line clipped function.
 *
 * Revision 1.2  2003/03/05 20:41:32  mju
 * Add GetTransLayerToDevice
 *
 * Revision 1.1  2002/09/19 17:08:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LAYERDC_H
#define	INC_GRE_LAYERDC_H

#ifndef	INC_GRE_VIEW_H
	#include <gre/view.h>
#endif

#ifndef	INC_GRE_GROUP_H
	#include <gre/group.h>
#endif

#ifndef	INC_GRE_LAYERVIEW_H
	#include <gre/layerview.h>
#endif

#ifndef	INC_GRE_LAYERSECTION_H
	#include <gre/layersection.h>
#endif

#ifndef	INC_GRE_GROUPVIEW_H
	#include <gre/groupview.h>
#endif

#ifndef	INC_GRE_3DSCENE_H
	#include <gre/3dscene.h>
#endif

#ifndef	INC_MI32_STEREOVIEW_H
	#include <mi32/stereoview.h>
#endif

#ifndef	INC_SML_CLASS_H
	#include <sml/class.h>
#endif

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
	#include <spatmod/imageresamplemethod.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace MGD {
	class COLORLOCK;
	}
namespace RVC {
	class RASTER;
	class NULLMASK;
	class STYLE;
	class STYLEPATTERN_BASE;
	class STYLEPATTERN_BITMAP;
	class STYLEPATTERN_HATCH;
	class STYLEPATTERN_LINE;
	class STYLEPATTERN_SYMBOL;
	}
namespace SPATMOD {
	class GEOREFERENCE;
	namespace IMAGE {
		class STAGE;
		class STAGE_AUTOPTR;
		}
	}
#endif


namespace GRE {
//===================================================================================================================
//! Layer drawing context.
//! Contains information needed to actually draw layer on a particular view.
class LAYERDC : public POLYLINE::CLIP {
	public:

		//! Construct using layer, view and drawing device.
		//! If transformation is provided, a private LAYERVIEW instance will be created with the
		//! specified transformation.  Typically this is done only in internal GRE implementations.
		LAYERDC (
			LAYER *layer,										//!< Layer, must not be NULL
			GRE::VIEW *view,									//!< View, should not be NULL
			MGD::DEVICE *pDevice = 0,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None,
			const TRANS2D_MAPGEN *pTransObjectToDevice = 0	//!< Optional object/device transformation
			);

		//! Construct using view, drawing device and transformation.
		//! Does not need a layer, but you need to calculate the transformation and pass it in.
		LAYERDC (
			GRE::VIEW *view,									//!< View, cannot be NULL
			MGD::DEVICE *pDevice,							//!< Drawing device, cannot be NULL
			const TRANS2D_MAPGEN& ObjectToDevice,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		// Construct using LAYERSECTION and RENDERTARGET.
		LAYERDC (
			GRE::LAYERSECTION *pLayerSection,			//!< Pointer to layer section, must remain valid until LAYERDC is destroyed
			RENDERTARGET content,							//!< Rendering target content
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Destructor.
		virtual ~LAYERDC ();

		//! Calculate dual displaced points from non-displaced for overhead stereo view.
		void CalcPointDisplacedDualZ (
			DPOINT3D& ptDevice,							//!< Point in device coordinates, Z value will be set from surface if needed
			DPOINT2D& ptDisplacedL,						//!< Left displaced point in device coordinates returned
			DPOINT2D& ptDisplacedR						//!< Right displaced point in device coordinates returned
			) const;

		//! Calculate left view displaced point from non-displaced for overhead stereo view.
		void CalcPointDisplacedLeftZ (
			DPOINT3D& ptDevice,							//!< Point in device coordinates, Z value will be set from surface if needed
			DPOINT2D& ptDisplaced						//!< Displaced point in device coordinates
			) const;

		//! Calculate right view displaced point from non-displaced for overhead stereo view.
		void CalcPointDisplacedRightZ (
			DPOINT3D& ptDevice,							//!< Point in device coordinates, Z value will be set from surface if needed
			DPOINT2D& ptDisplaced						//!< Displaced point in device coordinates
			) const;

		//! Determine if possible to draw layer.
		//! @return false if out of view or cannot draw based on 2D/3D view type.
		bool CanDrawLayer () const;

		//! Check if element is active/highlighted and get color if so.
		//! @return if element should be highlighted, false if not.
		bool CheckColorHighlight (
			COLOR& color,									//!< Color, will be updated if method returns true
			const RVC::ELEMENT& element,				//!< Element type/number
			const UNSIGNEDSET *pHighlightSet = 0	//!< Highlight set, if 0 will get from layer
			) const;

		//! Check if drawing flags require a color lock, set color and lock if so.
		//! @return true if lock was set, false if not
		bool CheckColorLock (
			MGD::COLORLOCK& colorlock				//!< Color lock passed/returned
			);

		//! Check if requires a color lock for specific element, set color and lock if so.
		bool CheckColorLock (
			MGD::COLORLOCK& colorlock,					//!< Color lock passed/returned
			const RVC::ELEMENT& element,				//!< Element type/number
			const UNSIGNEDSET *pHighlightSet = 0	//!< Highlight set, if 0 will get from layer
			);

		//! Check if requires a color lock, set color and lock if so.
		bool CheckColorLock (
			MGD::COLORLOCK& colorlock,					//!< Color lock passed/returned
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,								//!< Element number
			const UNSIGNEDSET *pHighlightSet = 0	//!< Highlight set, if 0 will get from layer
			) { return (CheckColorLock(colorlock,RVC::ELEMENT(ElemNum,ElemType),pHighlightSet)); }

		//! Check if need to override color based on drawing flags.
		bool CheckColorOverride (
			COLOR& color									//!< Override color returned
			) const;

		// Check if need to override color for specific element.
		bool GRE::LAYERDC::CheckColorOverride (
			COLOR& color,									//!< Override color returned
			const RVC::ELEMENT& element,				//!< Element type/number
			const UNSIGNEDSET *pHighlightSet = 0	//!< Highlight set, if 0 will get from layer
			) const;

		//! Clear all element "drawn" sets.
		void ClearAllDrawnSets ();

		// Translate and draw single line.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawLine (
			const RVC::ELEMENT& element,				//!< Element type/number
			const POLYLINE& lineObject,				//!< Line in layer object coordinates
			LINESTYLE& LineStyle,						//!< Line style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		// Translate and draw multiple lines.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawLines (
			const RVC::ELEMENT& element,				//!< Element type/number
			const POLYLINELIST& linesObject,			//!< Lines in layer object coordinates
			LINESTYLE& LineStyle,						//!< Line style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		DEPRECATED_MSG("Use DrawLineObject2D") int DrawLineClipped2D (
			const POLYLINE& lineObject					//!< Line in layer object coordinates
			) const { return (DrawLineObject2D(lineObject)); }

		//! Translate, clip and draw line in 3D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		DEPRECATED int DrawLineClipped3D (
			const POLYLINE& lineObject,				//!< Line in layer object coordinates
			bool DensifyOnSurface = true				//!< Densify to follow terrain surface if any
			) const;

		//! Draw line already in "device" coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawLineDevice2D (
			const POLYLINE& lineDevice					//!< Line in "device" coordinates
			) const;

		//! Draw line in layer object coordinates for 2D or 3D.
		int DrawLineObject (
			const POLYLINE& lineObject,				//!< Line in layer object coordinates
			LINESTYLE& LineStyle,						//!< Line style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Draw line in layer object coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawLineObject2D (
			const POLYLINE& lineObject					//!< Line in layer object coordinates
			) const;

		//! Draw line in layer object coordinates for 3D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawLineObject3D (
			const POLYLINE& lineObject,				//!< Line in layer object coordinates
			bool DensifyOnSurface = true				//!< Densify to follow terrain surface if any
			) const;

		//! Draw "managed point" symbols associated with specified element.
		//! @return true if found any symbols, false if not.
		bool GRE::LAYERDC::DrawManagedPointSymbols (
			const RVC::ELEMENT& element				//!< Element type/number
			);

		//! Translate and draw single point.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawPoint (
			const RVC::ELEMENT& element,				//!< Element type/number
			const DPOINT3D& ptObject,					//!< Point in layer object coordinates
			POINTSTYLE& PointStyle,						//!< Point style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		//! Draw single point in layer object coordinates for 2D or 3D.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawPointObject (
			const DPOINT3D& ptObject,					//!< Point in layer object coordinates
			POINTSTYLE& PointStyle,						//!< Point style
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for reading pattern
			const RVC::STYLEPATTERN_BASE *pPatternPoint = 0	//!< Optional pattern
			);

		//! Draw single point in 2D without translating or clipping, using previously set style.
		void DrawPointDevice2D (
			DPOINT3D& ptDevice						//!< Point in device coordinates, Z value will be set from surface if needed
			);

		//! Translate and draw single point in 2D, using previously set style.
		//! @return true if drawn, false if not.
		bool DrawPointObject2D (
			const DPOINT3D& ptObject					//!< Point in layer object coordinates
			);

		//! Translate and draw multiple points.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawPoints (
			const RVC::ELEMENT& element,				//!< Element type/number
			const POLYLINE& ptsObject,					//!< Points in layer object coordinates
			POINTSTYLE& PointStyle,						//!< Point style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		//! Draw/fill polygons from region in layer object coordinates for 2D or 3D.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawPolygons (
			const RVC::ELEMENT& element,				//!< Element type/number
			const REGION2D& rgnObject,					//!< Region in layer object coordinates
			POLYSTYLE& PolyStyle,						//!< Polygon style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		//! Draw/fill region already in device coordinates for 2D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawRegionDevice2D (
			const REGION2D& rgnDevice,					//!< Region in device coordinates
			POLYSTYLE& PolyStyle,						//!< Polygon style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Draw/fill region in layer object coordinates for 2D or 3D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawRegionObject (
			const REGION2D& rgnObject,					//!< Region in layer object coordinates
			POLYSTYLE& PolyStyle,						//!< Polygon style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Draw region outline from layer object coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawRegionObject2D (
			const REGION2D& rgnObject					//!< Region in layer object coordinates
			) const;

		//! Draw region outline from layer object coordinates for 3D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawRegionObject3D (
			const REGION2D& rgnObject					//!< Region in layer object coordinates
			) const;

		//! Draw segment in device coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSegmentDevice2D (
			const DPOINT3D& ptDevice1,					//!< First point in layer object coordinates
			const DPOINT3D& ptDevice2					//!< Second point in layer object coordinates
			) const;

		//! Draw segment in device coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSegmentDevice2D (
			double p1x,
			double p1y,
			double p2x,
			double p2y
			) const { return (DrawSegmentDevice2D(DPOINT2D(p1x,p1y),DPOINT2D(p2x,p2y))); }

		//! Clip and draw segment in 3D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSegment3D (
			const DPOINT3D& ptObject1,					//!< First point in layer object coordinates
			const DPOINT3D& ptObject2					//!< Second point in layer object coordinates
			) const;

		//! Draw/fill subregion in layer object coordinates for 2D or 3D.
		//! @return TRUE if in visible extents, FALSE if not, < 0 if error.
		int DrawSubRegion (
			const RVC::ELEMENT& element,				//!< Element type/number
			const SUBREGION2D& srObject,				//!< Subregion in layer object coordinates
			POLYSTYLE& PolyStyle,						//!< Polygon style, may be altered if necessary
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			DRAWFLAGS drawflags = DRAWFLAG_None,	//!< Additional drawing flags
			int StyleScriptSeg = -1						//!< Script segment if styling by script, < 0 if not doing so
			);

		//! Draw/fill subregion already in device coordinates for 2D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSubRegionDevice2D (
			const SUBREGION2D& srDevice,				//!< Subregion in device coordinates
			POLYSTYLE& style,								//!< Style for fill and/or border
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Draw subregion outline in device coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSubRegionDevice2D (
			const SUBREGION2D& srDevice				//!< Subregion in device coordinates
			);

		//! Draw/fill subregion in layer object coordinates for 2D.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSubRegionObject (
			const SUBREGION2D& srObject,				//!< Subregion in layer object coordinates
			POLYSTYLE& style,								//!< Style for fill and/or border
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Draw subregion outline in 3D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int DrawSubRegionObject3D (
			const SUBREGION2D& srObject				//!< Subregion in layer object coordinates
			) const;

		//! Draw "flat" wireframe of specified region.
		ERRVALUE DrawWireframeFlat (
			const REGION2D& rgnObject					//!< Region in layer object coordinates
			) const;

		//! Fill 3D flat polygon.
		//! @return true if was visible, false if not.
		bool Fill3dFlatPolygon (
			const DPOINT3D *ptsObject,					//!< Polygon vertices in layer object coordinates
			INT32 numPoints								//!< Number of vertices
			) const;

		//! Fill 3D triangle.
		//! @return true if was visible, false if not.
		bool Fill3dTriangle (
			const DPOINT3D *tp[3]						//!< Pointers to triangle vertices with layer object coordinates
			) const;

		//! Fill subregion in device coordinates for 2D using currently set style.
		//! @return TRUE if visible, FALSE if not, < 0 if error.
		int FillSubRegionDevice2D (
			const SUBREGION2D& srDevice,				//!< Subregion in device coordinates
			bool DoBorder = false
			);

		//! Get depth scaling, multiplied by depth to get drawing scale.
		double Get3dDepthScale (
			) const { return (m_pLayerView->Get3dDepthScale()); }

		//! Get 3D scene containing 3D transformations.
		const GRE::SCENE3D& Get3dScene (
			) const { return (m_Scene3D); }

		//! Get 3D scene containing 3D transformations.
		GRE::SCENE3D& Get3dScene (
			) { return (m_Scene3D); }

		//! Get increment in layer relative to once cell in surface layer.
		double Get3dSurfaceIncrement (
			) const { return (m_pLayerView->Get3dSurfaceIncrement()); }

		//! Get 3D stereo view setting.
		STEREOVIEW Get3dStereoView (
			) const { return (m_Scene3D.GetStereoView()); }

		//! Get 3D Z scaling.
		double Get3dZScale (
			) const { return (m_pLayerView->Get3dZScale()); }

		//! Get clipping region in device coordinates.
		const REGION2D& GetClipRegion (
			) const { return (m_ClipRegion); }

		//! Get design scale.
		double GetDesignScale (
			) const { return (m_DesignScale); }

		//! Get device rectangle to be used for rendering.
		const LRECT2D& GetDeviceRectUsed () const
			{ return (m_pLayerView->GetDeviceRectUsed()); }

		//! Get drawing flags.
		GRE::DRAWFLAGS GetDrawFlags (
			) const { return (m_drawflags); }

		//! Get drawing device context.
		MGD::CONTEXT* GetDrawingContext (
			) const { return (m_ggc); }

		//! Get drawing device.
		MGD::DEVICE* GetDrawingDevice (
			) const;

		//! Get scale at which layer is currently being drawn.
		double GetDrawingScale (
			) const { return (m_DrawingScale); }

		//! Get pointer to 'drawn set' based on specified element type and current target device.
		//! @return Pointer to UNSIGNEDSET, NULL if none available.
		UNSIGNEDSET* GetElemDrawnSet (
			ELEMTYPE ElemType = ELEMTYPE_All
			) const;

		//! Get layer.
		LAYER* GetLayer (
			) const { return (m_layer); }

		//! Get LAYERSECTION (const).
		const GRE::LAYERSECTION* GetLayerSection (
			) const { return (m_pLayerSection); }

		//! Get LAYERSECTION (non-const).
		GRE::LAYERSECTION* GetLayerSection (
			) { return (m_pLayerSection); }

		//! Get LAYERVIEW (const).
		const GRE::LAYERVIEW* GetLayerView (
			) const { return (m_pLayerView); }

		//! Get LAYERVIEW (non-const).
		GRE::LAYERVIEW* GetLayerView (
			) { return (m_pLayerView); }

		//! Determine whether to not draw "active" element.
		bool GetNoDrawActive (
			) const { return (m_NoDrawActive); }

		//! Get the clip rectangle in object coordinates
		const DRECT2D& GetObjectClipRect (
			) const { return (m_ObjClipRect); }

		//! Get pixel size in millimeters.
		const DPOINT2D& GetPixelSizeMM (
			) const { return (m_PixelSizeMM); }

		//! Get target based on current drawing device.
		GRE::RENDERTARGET GetRenderTarget (
			) const { return (m_RenderTarget); }

		//! Get rotation angle.
		double GetRotationAngle (
			) const { return (m_RotationAngle); }

		//! Get rotation cosine.
		double GetRotationCos (
			) const { return (m_pGroupView->GetRotationCos()); }

		//! Get rotation sine.
		double GetRotationSin (
			) const { return (m_pGroupView->GetRotationSin()); }

		//! Get terrain shift values for stereo rendering.
		const STEREOSHIFT& GetStereoShift () const
			{ return (m_pGroupView->GetStereoShift()); }

		//! Determine if first row(column) of device is for right eye in interlaced stereo rendering.
		bool GetStereoInterlaceStartRight (
			) const { return (m_pGroupView->GetStereoInterlaceStartRight()); }

		//! Get surface layer for 3D draping.
		LAYER_SURFACE* GetSurfaceLayer (
			) const { return (m_pLayerView->GetSurfaceLayer()); }

		//! Get surface Z coordinates for POLYLINE.
		ERRVALUE GetSurfaceZ (
			POLYLINE& plObject						//!< 3D polyline in layer object coordinates
			) const;

		//! Get surface Z coordinates for array of points.
		ERRVALUE GetSurfaceZ (
			DPOINT3D *ptsObject,						//!< Array of points with X/Y coordinates in layer associated with context
			INT32 NumPoints							//!< Number of points
			) const;

		//! Get surface Z coordinate for point.
		ERRVALUE GetSurfaceZ (
			const DPOINT2D& spoint,					//!< Point in surface object coordinates
			double& z,									//!< Z value returned
			bool DoSearch								//!< Search around point if actual cell is null
			) const;

		//! Get transformation from 'layer' to 'device' coordinates.
		const TRANS2D_MAPGEN& GetTransLayerDevice (
			) const;

		//! Get transformation from 'layer' to 'surface' coordinates.
		const TRANS2D_MAPGEN& GetTransLayerSurface (
			) const { return (m_pLayerView->GetTransLayerSurface()); }

		GRE::VIEW* GetView (
			) const { return (m_view); }

		//! Determine if has surface layer.
		bool HasSurfaceLayer (
			) const { return (m_pLayerView->GetSurfaceLayer() != 0); }

		//! Draw image in 2D.
		ERRVALUE ImageDraw2D (
			SPATMOD::IMAGE::STAGE *pStageInput,
			const LRECT2D* SectionRect = 0,		//!< Section area to process
			bool bForceAlphaBlend = false		//!< Force context for perform alpha blending
			) const;

		//! Insert image filter for resampling to display.
		ERRVALUE ImageInsertResampleFilter (
			SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage,			//!< Image stage to insert into
			SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod = SPATMOD::IMAGE::RESAMPLEMETHOD_Nearest	//!< Resampling method
			) const;

		//! Insert image filter for resampling to display.
		ERRVALUE ImageInsertResampleFilterStereo2D (
			SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage,			//!< Image stage to insert into
			SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStageTerrain,	//!< Stage pointer for terrain, will be returned detached
			SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod = SPATMOD::IMAGE::RESAMPLEMETHOD_Nearest	//!< Resampling method
			) const;

		//! Insert image filter for zooming to display.
		ERRVALUE ImageInsertZoomFilter (
			SPATMOD::IMAGE::STAGE_AUTOPTR& PtrStage,		//!< Image stage to insert into
			double *pZoomUsed = 0								//!< Pointer to zoom value used returned, NULL if don't need
			) const;

		//! Determine if drawing in 2D or 3D.
		bool Is3D () const
			{ return (m_pLayerView->Is3D()); }

		//! Determine if layer is 'hidden' due to scale or other visibility setting.
		bool IsHidden () const
			{ return (m_IsHidden); }

		//! Determine if is managing point symbols.
		bool IsManagingPointSymbols () const
			{ return (m_layer->IsManagingPointSymbols() && !m_layer->IsEditable() && !m_IsStereo && !Is3D()); }

		//! Determine if drawing "stereo" view.
		bool IsStereo () const
			{ return (m_IsStereo); }

		//! Determine if surface (layer or flat) is being used.
		bool IsSurfaceUsed () const
			{ return (m_pLayerView->IsSurfaceUsed()); }

		//! Send notification before drawing element.
		void NotifyDrawElementBegin (
			const RVC::ELEMENT& element
			) const { if (m_NeedNotifyDrawElement) m_view->NotifyDrawElementBegin(m_layer,element); }

		//! Send notification before drawing element.
		void NotifyDrawElementBegin (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum
			) const { if (m_NeedNotifyDrawElement) m_view->NotifyDrawElementBegin(m_layer,RVC::ELEMENT(ElemNum,ElemType)); }

		//! Send notification after drawing element.
		void NotifyDrawElementEnd (
			const RVC::ELEMENT& element
			) const { if (m_NeedNotifyDrawElement) m_view->NotifyDrawElementEnd(m_layer,element); }

		//! Send notification after drawing element.
		void NotifyDrawElementEnd (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum
			) const { if (m_NeedNotifyDrawElement) m_view->NotifyDrawElementEnd(m_layer,RVC::ELEMENT(ElemNum,ElemType)); }

		//! Set whether current element is scaled when rendered in 3D.
		void Set3dElemScaled (
			bool IsScaled
			) { m_ScaleToMap3D = IsScaled; }

		void Set3dStereoView (
			STEREOVIEW StereoView
			);

		//! Set drawing flags.
		void SetDrawFlags (
			GRE::DRAWFLAGS drawflags
			);

		//! Set drawing device.
		void SetDrawingDevice (
			MGD::DEVICE *pDevice
			);

		//! Set render target.
		void SetRenderTarget (
			RENDERTARGET RenderTarget
			) { m_RenderTarget = RenderTarget; }

		//! Set for drawing left stereo view.
		//! @return false if not using stereo, true if so.
		bool SetStereoLeft ();

		//! Set for drawing right stereo view.
		//! @return false if not using stereo, true if so.
		bool SetStereoRight ();

		//! Set style for border, reading pattern if necessary.
		void SetStyleBorder (
			const POLYSTYLE& BorderStyle,				//!< Border style
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Set style for filling, reading pattern if necessary.
		void SetStyleFill (
			POLYSTYLE& FillStyle,						//!< Fill style
			RVC::STYLE *pObjStyle = 0					//!< Style object if needed for pattern
			);

		//! Set style for line, reading pattern if necessary, with option to check for multipass.
		//! @return false if should skip on this pass, true if should draw or if TestMultiPass is false.
		bool SetStyleLine (
			LINESTYLE& LineStyle,						//!< Line style
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for pattern
			bool TestMultiPass = false					//!< Perform multipass test before setting style
			);

		// Set style for point, reading pattern if necessary.
		void SetStylePoint (
			POINTSTYLE& PointStyle,						//!< Point style
			RVC::STYLE *pObjStyle = 0,					//!< Style object if needed for reading pattern
			const RVC::STYLEPATTERN_BASE *pPatternPoint = 0	//!< Optional pattern
			);

		//! Set "style" script context.
		void SetStyleScriptContext (
			GRE::SCRIPTCONTEXT_STYLE *pStyleSC
			) { m_pStyleSC = pStyleSC; }

		//! Determine if rendering stereo and setup if needed.
		ERRVALUE SetupStereo ();

		//! Translate 3D point from layer/object to display coordinates.
		//! @return TRUE if point visible, FALSE if not, < 0 if error.
		int TransPoint3D (
			DPOINT3D& pt3d								//!< Point initially in layer/object coordinates, returned in display coordinates with canonical depth
			) const;

		//! Translate POLYLINE to device coordinates.
		//! Will perform terrain drape and stereo displacement if needed.
		ERRVALUE TransToDevice (
			POLYLINE& polyline						//!< Line initially in layer object coordinates
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static SML::AUTOCLASS_NOALLOC_T<LAYERDC> s_SML_AutoClass;

		static void SML_Register ();

		GRE::GROUPVIEW *m_pGroupView;
		GRE::LAYERVIEW *m_pLayerView;
		GRE::LAYERSECTION *m_pLayerSection;
		LAYER *m_layer;
		GRE::VIEW *m_view;
		MGD::CONTEXT *m_ggc;									// Generic graphic context
		GRE::DRAWFLAGS m_drawflags;						// Drawing flags
		REGION2D m_ClipRegion;								// Current clipping region in device coordinates
		DRECT2D m_ObjClipRect;								// Clipping rectangle in object coordinates
		DPOINT2D m_PixelSizeMM;								// Device pixel size in millimeters
		double m_DesignScale;								// Mirrored from LAYERVIEW for SML access
		double m_DrawingScale;
		double m_RotationAngle;								// Mirrored from GROUPVIEW for SML access
		bool m_OwnGroupView;
		bool m_OwnLayerView;
		bool m_IsHidden;
		bool m_NoDrawActive;									// Don't draw "active" element
		bool m_ScaleToMap3D;									// Set if 3D scaling to map
		bool m_NeedNotifyDrawElement;
		bool m_IsStereo;
		bool m_StereoUseMasks;
		GRE::SCENE3D m_Scene3D;
		GRE::RENDERTARGET m_RenderTarget;
		const ZVIEW *m_pZView;
		RVC::STYLEPATTERN_BITMAP *m_pLastPatternBitmap;
		RVC::STYLEPATTERN_HATCH *m_pLastPatternHatch;
		RVC::STYLEPATTERN_LINE *m_pLastPatternLine;
		RVC::STYLEPATTERN_SYMBOL *m_pLastPatternSymbol;
		GRE::SCRIPTCONTEXT_STYLE *m_pStyleSC;
		INT32 m_LastPatternNumBitmap;
		INT32 m_LastPatternNumHatch;
		INT32 m_LastPatternNumLine;
		INT32 m_LastPatternNumSymbol;
		UINT8 m_StereoMaskLeftRGB[3];
		UINT8 m_StereoMaskRightRGB[3];

		void AddManagedPointSymbol (LABELPLACEMENT *pLabelPlacement, const RVC::ELEMENT& element, const POINTSTYLE& PointStyle, const RVC::STYLEPATTERN_BASE *pPattern, const DPOINT3D& ptDevice) const;
		void ImageCalcDeviceGeoreference (SPATMOD::GEOREFERENCE& Georeference) const;
		const RVC::STYLEPATTERN_BASE* ReadFillPattern (POLYSTYLE& PolygonStyle, const RVC::STYLE *pObjStyle);
		const RVC::STYLEPATTERN_BASE* ReadLinePattern (LINESTYLE& LineStyle, const RVC::STYLE *pObjStyle);
		const RVC::STYLEPATTERN_BASE* ReadPointPattern (POINTSTYLE& PointStyle, const RVC::STYLE *pObjStyle);
		void SetupForDrawFlags ();
		void SetupForStereoView ();
		ERRVALUE SetupStereoIn2D ();

		// Not implemented.
		LAYERDC (const LAYERDC&);
		LAYERDC& operator= (const LAYERDC&);
		
		virtual ERRVALUE OnClip (const POLYLINE& PolyLine);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LAYERDC_H
