/**
 * \file gre/viewplace.h
 * \brief GRE VIEWPLACE class definitions
 *
 * \if NODOC
 * $Id: viewplace.h_v 1.3 2011/10/20 13:34:07 mju Exp $
 *
 * $Log: viewplace.h_v $
 * Revision 1.3  2011/10/20 13:34:07  mju
 * Remove redundant hdrs.
 *
 * Revision 1.2  2007/10/15 16:00:31  mju
 * Add equality tests.
 *
 * Revision 1.1  2007/10/05 14:17:46  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_VIEWPLACE_H
#define	INC_GRE_VIEWPLACE_H

#ifndef INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif

#ifndef INC_MI32_POINT_H
	#include <mi32/point.h>
#endif


namespace GRE {
//===================================================================================================================

//! Simple container for specifying "place" being viewed.
//! Typically this will specify a center point, CRS and scale.  However, may also specify a "width" to view using
//! any linear unit.
class VIEWPLACE {
	public:

		VIEWPLACE ();

		~VIEWPLACE ();

		//! Get CRS for point.
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CRS); }

		//! Get point.
		const DPOINT2D& GetPoint (
			) const { return (m_Point); }

		//! Get size value.
		double GetSize (
			) const { return (m_Size); }

		//! Get size unit.
		const UOM::UNIT GetSizeUnit (
			) const { return (m_SizeUnit); }

		//! Test for equality.
		bool IsEqual (
			const VIEWPLACE& rhs
			) const;

		//! Set point and CRS.
		void SetPoint (
			const SPATREF::COORDREFSYS& CoordRefSys,	//!< CRS for point
			const DPOINT2D& Point				//!< Point in specified CRS
			) { m_CRS = CoordRefSys; m_Point = Point; }

		//! Set point.
		void SetPoint (
			const DPOINT2D& Point				//!< Point in previously specified CRS
			) { m_Point = Point; }

		//! Set size and unit.
		void SetSize (
			UOM::UNIT& SizeUnit,					//!< Unit for size, typically has MEASURE::TYPE_Scale or TYPE_Length
			double Size								//!< Value for size, typically indicates scale or horizontal extent
			) { m_SizeUnit = SizeUnit; m_Size = Size; }

		//! Set size.
		void SetSize (
			double Size								//!< Value for size, typically indicates scale or horizontal extent
			) { m_Size = Size; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
			SPATREF::COORDREFSYS m_CRS;
			DPOINT2D m_Point;
			UOM::UNIT m_SizeUnit;
			double m_Size;
		#endif
	};

inline bool operator== (
	const VIEWPLACE& lhs,
	const VIEWPLACE& rhs
	) {
	return (lhs.IsEqual(rhs));
	}

inline bool operator!= (
	const VIEWPLACE& lhs,
	const VIEWPLACE& rhs
	) {
	return (!lhs.IsEqual(rhs));
	}

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_VIEWPLACE_H
