/**
 * \file mgui/combobox.h
 * \brief MGUI::COMBOBOX definitions
 *
 * \if NODOC
 * $Id: combobox.h_v 1.39 2012/01/27 18:40:02 mju Exp $
 *
 * $Log: combobox.h_v $
 * Revision 1.39  2012/01/27 18:40:02  mju
 * Remove mfc stuff.
 *
 * Revision 1.38  2011/11/22 13:46:51  mju
 * Fix dox dependencies.
 *
 * Revision 1.37  2011/09/01 17:57:32  mju
 * Exclude macro from dox.
 *
 * Revision 1.36  2010/12/13 17:19:19  mju
 * Add iniRead with default ID.
 *
 * Revision 1.35  2010/11/12 15:59:24  mju
 * Add addUnitItem.
 *
 * Revision 1.34  2009/02/26 13:30:55  mju
 * Add setSelectedPrevious.
 * Add 'notify' parm to setSelected...
 *
 * Revision 1.33  2008/05/01 13:43:45  mju
 * Add m_maxwidth member.
 *
 * Revision 1.32  2007/03/20 12:46:01  mju
 * Default dropdown list height to 12.
 *
 * Revision 1.31  2007/02/17 00:21:38  scowan
 * Added virtual method called after create.
 *
 * Revision 1.30  2007/01/12 15:38:08  mju
 * Remove unused deprecated methods having char* labels.
 *
 * Revision 1.29  2007/01/05 23:26:48  scowan
 * Removed form_combobox_t.
 *
 * Revision 1.28  2006/10/10 15:53:49  scowan
 * Since we are returning non-pointer versions of mistring, we need to include it here.
 *
 * Revision 1.27  2006/10/06 13:32:28  mju
 * Use inidefns hdr.
 *
 * Revision 1.26  2006/10/02 13:25:35  mju
 * Remove ctrl_combobox_t as unused.
 *
 * Revision 1.25  2006/07/17 13:55:02  mju
 * Remove setEnabled override for form so uses base implementation.  This results in dimming current combobox selection consistently.
 *
 * Revision 1.24  2006/04/07 20:14:51  mju
 * Exclude entirely if not use_mgui.
 *
 * Revision 1.23  2006/01/25 14:29:43  mju
 * Deprecate methods using listitemdef.
 *
 * Revision 1.22  2006/01/09 21:55:19  mju
 * Exclude deprecated methods when compile for no_deprecated.
 *
 * Revision 1.21  2005/11/14 15:00:31  mju
 * No deprecation warnings within deprecated inlines.
 *
 * Revision 1.20  2005/11/14 14:27:42  mju
 * Exclude templates if no_deprecated.
 * Deprecation warnings for char* labels.
 *
 * Revision 1.19  2005/10/06 21:24:35  mju
 * Forward labelstyle parm in template create.
 *
 * Revision 1.18  2005/06/27 14:09:04  mju
 * Still allow template for no_deprecated so sdedit builds.
 *
 * Revision 1.17  2005/06/27 13:50:06  mju
 * Add deprecation warnings for template and exclude when compile for no_deprecated.
 *
 * Revision 1.16  2005/06/24 14:56:54  mju
 * Add setDelegate method to form.
 *
 * Revision 1.15  2005/06/23 21:59:44  mju
 * Support delegates.
 *
 * Revision 1.14  2005/03/31 16:57:07  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.13  2004/01/02 22:21:19  mju
 * Need to override setenabled for form too.
 *
 * Revision 1.12  2003/12/22 14:16:03  mju
 * Make SetEnabled public.
 *
 * Revision 1.11  2003/12/19 22:07:57  mju
 * Override setenabled for X.
 *
 * Revision 1.10  2003/11/17 18:10:22  mju
 * Add combobox::sizeToContentWidth.
 *
 * Revision 1.7  2003/09/11 17:05:20  mju
 * Use cast in IniWrite to resolve ambiguity.
 *
 * Revision 1.6  2003/09/10 21:15:25  mju
 * Use ID for id.
 *
 * Revision 1.4  2003/09/05 17:09:43  mju
 * AddItem using const MISTRING&.
 *
 * Revision 1.1  2003/01/16 21:13:02  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_COMBOBOX_H
#define	INC_MGUI_COMBOBOX_H

#ifndef INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef INC_MI32_INIDEFNS_H
#include <mi32/inidefns.h>
#endif

namespace MGUI {

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLIST;
#endif


//===================================================================================================================
//! ComboBox control.
//! At present this is implemented as an OptionMenu for X.
class CTRL_COMBOBOX : public MGUI::CTRL {
	public:

		enum STYLE {
			STYLE_Default =	0x00,
			STYLE_Sorted =		0x01
			};

		//! Constructor.
		CTRL_COMBOBOX (
			);

		//! Destructor.
		virtual ~CTRL_COMBOBOX (
			);

		//! Add single item with MISTRING or TEXTID.
		//! If control was created with STYLE_Sorted then the combo will be resorted after the item
		//! is added, otherwise the item will be added to the end of the list.
		//! @return Zero-based index of item added or error < 0.
		int AddItem (
			ID id,											//!< ID to attach to item
			const MISTRING& string,						//!< String to show in list
			bool resize = true							//!< Attempt to resize if needed to fit new item string
			);

		//! Add item to combobox for unit selection.
		//! Item ID will be equal to UnitID.
		void AddUnitItem (
			int MeasureType,								//!< Measurement type
			int UnitID										//!< Unit ID
			);

		//! Add items to combobox for unit selection.
		//! Item IDs will be equal to UnitIDs.
		void AddUnitItems (
			int MeasureType,								//!< Measurement type
			int DftUnitID = 0								//!< Default unit ID
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int listheight = 12,							//!< Maximum list height when dropped down in characters
			int width = 0,									//!< Width in 'typical' characters, 0 for default
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			STYLE style = STYLE_Default
			);

		//! Delete all items from control.
		void DeleteAllItems (
			);

		//! Delete item from list given index.
		void DeleteItem (
			int index									//!< Zero-based index of item to delete
			);

		//! Delete item from list given ID.
		void DeleteItemID (
			ID id										//!< ID of item to delete
			);

		//! Retrieve number of items in the list.
		int GetCount (
			) const;

		//! Get index to item from ID.
		//! @return Zero-based index or -1 if no matching item found.
		int GetIndexFromID (
			ID id										//!< ID of item to retrieve index of
			) const;

		//! Get ID associated with list item.
		//! @return ID specified when item was added.
		ID GetItemID (
			int index									//!< Zero-based index of item
			) const;

		//! Get text string associated with list item.
		//! @return String containing text, will be empty if invalid item.
		MISTRING GetItemString (
			int index									//!< Zero-based index of item
			) const;

		//! Get currently selected item.
		//! @return Zero-based index of selected item or -1 if none selected.
		int GetSelectedItem (
			) const;

		//! Get ID of currently selected item.
		//! @return ID or -1 if item was not found or none selected.
		ID GetSelectedItemID (
			) const;

		//! Read setting from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnSelection() if control has already been created
			);

		//! Read setting from INI file with default ID.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			ID dftid,									//!< Default ID
			bool notify									//!< Call OnSelection() if control has already been created
			);

		//! Write currently selected item ID to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const { ::IniWrite(IniHandle,IniGroup,IniField,static_cast<INT32>(GetSelectedItemID())); }

		//! Insert item at specified position.
		//! Unlike the AddItem method this does not cause the list to be resorted.
		//! @return Zero-based index of item added or error < 0.
		int InsertItem (
			int index,									//!< Zero-based index of position at which item should be inserted
			ID id,										//!< ID to attach to item
			const MISTRING& string					//!< String to show in combo
			);

		//! Determine if specified item is selected.
		bool IsItemSelected (
			int index									//!< Zero-based index of item
			) const;

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnSelection = delegate; }

		//! Set whether control is enabled.
		virtual void SetEnabled (
			bool enabled = true
			);

		//! Select item using zero-based index.
		//! If item is not visible the list will be scrolled to make it visible.
		void SetSelectedItem (
			int index,								//!< Zero-based index of item, -1 to have no item selected
			bool notify = false					//!< Notify delegate if selection changed
			);

		//! Select item using item ID.
		//! If item does not exist then no item will be selected.
		//! If item is not visible the list will be scrolled to make it visible.
		void SetSelectedItemID (
			ID id,									//!< ID of item to select
			bool notify = false					//!< Notify delegate if selection changed
			);

		//! Set selected item to item prior to current selection.
		//! Intended to be used in delegate to revert selection on failure.
		void SetSelectedPrevious (
			bool notify = false					//!< Notify delegate if selection changed
			);

		//! Set whether items are to be sorted or not.
		void SetSorted (
			bool sorted = true
			);

		//! Resize control to be able to show widest item.
		void SizeToContentWidth (
			);

	private:
		//! Used to fill in the items of the combobox for a derived class
		virtual void v_CreateEnd ();
		
	#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnSelection;
		Widget m_text;
		Widget m_arrowpb;
		XMLIST *m_pxmlist;
		Widget m_listshell;
		UINT16 m_listheight;
		UINT16 m_maxwidth;
		int m_CurrentIndex;
		int m_PreviousIndex;

		static void CB_ArrowActivate (Widget, CTRL_COMBOBOX*, void*);
		static void CB_ListBrowseSelect (Widget, CTRL_COMBOBOX*, void*);
		static void EH_Shell (Widget, CTRL_COMBOBOX*, XEvent*, Boolean*);

		void OnSelection ();
		void PopdownShell ();
		void PopupShell ();
		bool UpdateCurrentIndex ();

		CTRL_COMBOBOX (const CTRL_COMBOBOX&);
		CTRL_COMBOBOX& operator= (const CTRL_COMBOBOX&);
	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_COMBOBOX::STYLE)
#endif


//===================================================================================================================
//! Form containing label and Combobox control.
class FORM_COMBOBOX : public MGUI::FORM_COMPOSITE {
	public:

		//! Default constructor.
		FORM_COMBOBOX (
			);

		//! Destructor.
		virtual ~FORM_COMBOBOX (
			);

		//! Add single item with MISTRING or TEXTID.
		//! If control was created with STYLE_Sorted then the combo will be resorted after the item
		//! is added, otherwise the item will be added to the end of the list.
		//! @return Zero-based index of item added or error < 0.
		int AddItem (
			ID id,											//!< ID to attach to item
			const MISTRING& string,						//!< String to show in list
			bool resize = true							//!< Attempt to resize if needed to fit new item string
			) { return (m_combobox.AddItem(id,string,resize)); }

		//! Add item to combobox for unit selection.
		//! Item ID will be equal to UnitID.
		void AddUnitItem (
			int MeasureType,								//!< Measurement type
			int UnitID										//!< Unit ID
			) { m_combobox.AddUnitItem(MeasureType,UnitID); }

		//! Add items to combobox for unit selection.
		//! Item IDs will be equal to UnitIDs.
		void AddUnitItems (
			int MeasureType,								//!< Measurement type
			int DftUnitID = 0								//!< Default unit ID
			) { m_combobox.AddUnitItems(MeasureType,DftUnitID); }

		//! Create form with MISTRING label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			int listheight = 12,							//!< Maximum list height when dropped down in characters
			int width = 0,									//!< Width in 'typical' characters, 0 for default
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			MGUI::CTRL_COMBOBOX::STYLE style = MGUI::CTRL_COMBOBOX::STYLE_Default,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Delete all items from control.
		void DeleteAllItems (
			) { m_combobox.DeleteAllItems(); }

		//! Delete item from list given index.
		void DeleteItem (
			int index									//!< Zero-based index of item to delete
			) { m_combobox.DeleteItem(index); }

		//! Delete item from list given ID.
		void DeleteItemID (
			ID id										//!< ID of item to delete
			) { m_combobox.DeleteItemID(id); }

		//! Get reference to CTRL_COMBOBOX in form.
		CTRL_COMBOBOX& GetComboBox (
			) { return (m_combobox); }

		//! Get reference to CTRL_COMBOBOX in form.
		const CTRL_COMBOBOX& GetComboBox (
			) const { return (m_combobox); }

		//! Retrieve number of items in the list.
		int GetCount (
			) const { return (m_combobox.GetCount()); }

		//! Get index to item from ID.
		//! @return Zero-based index or -1 if no matching item found.
		int GetIndexFromID (
			ID id										//!< ID of item to retrieve index of
			) const { return (m_combobox.GetIndexFromID(id)); }

		//! Get ID associated with list item.
		//! @return ID specified when item was added.
		ID GetItemID (
			int index									//!< Zero-based index of item
			) const { return (m_combobox.GetItemID(index)); }

		//! Get text string associated with list item.
		//! @return String containing text, will be empty if invalid item.
		MISTRING GetItemString (
			int index									//!< Zero-based index of item
			) const { return (m_combobox.GetItemString(index)); }

		//! Get label control if any.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Get currently selected item.
		//! @return Zero-based index of selected item or -1 if none selected.
		int GetSelectedItem (
			) const { return (m_combobox.GetSelectedItem()); }

		//! Get ID of currently selected item.
		//! @return ID or -1 if item was not found or none selected.
		ID GetSelectedItemID (
			) const { return (m_combobox.GetSelectedItemID()); }

		//! Insert item at specified position.
		//! Unlike the AddItem method this does not cause the list to be resorted.
		//! @return Zero-based index of item added or error < 0.
		int InsertItem (
			int index,									//!< Zero-based index of position at which item should be inserted
			ID id,									//!< ID to attach to item
			const MISTRING& string					//!< String to show in combo
			) { return (m_combobox.InsertItem(index,id,string)); }

		//! Determine if specified item is selected.
		bool IsItemSelected (
			int index									//!< Zero-based index of item
			) const { return (m_combobox.IsItemSelected(index)); }

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_combobox.SetDelegateOnSelection(delegate); }

		//! Select item using zero-based index.
		//! If item is not visible the list will be scrolled to make it visible.
		void SetSelectedItem (
			int index,									//!< Zero-based index of item, -1 to have no item selected
			bool notify = false					//!< Notify delegate if selection changed
			) { m_combobox.SetSelectedItem(index,notify); }

		//! Select item using item ID.
		//! If item does not exist then no item will be selected.
		//! If item is not visible the list will be scrolled to make it visible.
		void SetSelectedItemID (
			ID id,										//!< ID of item to select
			bool notify = false					//!< Notify delegate if selection changed
			) { m_combobox.SetSelectedItemID(id,notify); }

		//! Set selected item to item prior to current selection.
		//! Intended to be used in delegate to revert selection on failure.
		void SetSelectedPrevious (
			bool notify = false					//!< Notify delegate if selection changed
			) { m_combobox.SetSelectedPrevious(notify); }

		//! Set whether items are to be sorted or not.
		void SetSorted (
			bool sorted = true
			) { m_combobox.SetSorted(sorted); }

	private:
		//! Used to fill in the items of the combobox for a derived class
		virtual void v_CreateEnd ();
		
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MGUI::CTRL_COMBOBOX m_combobox;
		MGUI::CTRL_LABEL m_label;

		FORM_COMBOBOX (const FORM_COMBOBOX&);
		FORM_COMBOBOX& operator= (const FORM_COMBOBOX&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_COMBOBOX_H
