/**
 * \file mgui/dlgstatus.h
 * \brief MGUI::DLG_STATUS definitions.
 *
 * \if NODOC
 * $Id: dlgstatus.h_v 1.8 2011/11/22 21:32:27 mju Exp $
 *
 * $Log: dlgstatus.h_v $
 * Revision 1.8  2011/11/22 21:32:27  mju
 * Fix dox dependencies.
 *
 * Revision 1.7  2007/12/05 18:45:03  mju
 * Add mgd context member so don't have to construct every time update.
 *
 * Revision 1.6  2007/11/16 18:52:15  scowan
 * Added timer for multi-thrreading work.
 *
 * Revision 1.5  2007/11/14 22:04:00  mju
 * Add onPaint handler.
 *
 * Revision 1.4  2007/11/09 18:34:38  mju
 * Now working.
 *
 * Revision 1.3  2007/11/08 13:46:52  mju
 * Remove unneeded member.
 *
 * Revision 1.2  2007/11/02 21:47:41  mju
 * Override reportTimerValue.
 *
 * Revision 1.1  2007/11/02 15:48:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGSTATUS_H
#define	INC_MGUI_DLGSTATUS_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif

#ifndef INC_MGUI_TIMER_H
#include <mgui/timer.h>
#endif

#ifndef INC_MI32_MISTATUS_H
#include <mi32/mistatus.h>
#endif

namespace MGUI {
//=====================================================================================================================

class DLG_STATUS : public MISTATUSLOGGER, private MGUI::DLGSHELL {
	public:

		//! Dialog style flags.
		enum DLGSTYLE {
			DLGSTYLE_Default =			0x0000,
			DLGSTYLE_DisableCancel =	0x0001,		//!< Disable cancel button
			DLGSTYLE_DelayPopup =		0x0002,
			DLGSTYLE_EnableThreading =	0x0004,		//!< Enable the status dialog to handle status from multiple threads
			};

		//! Used for MISTATUSMANAGER::SetCreator to set as default status manager creator.
		static MISTATUSMANAGER* CreateManager ();

		//! Constructor.
		DLG_STATUS ();

		//! Destructor.
		~DLG_STATUS ();

		//! Close dialog.
		void Close (
			) { DLGSHELL::Close(); }

		//! Create dialog.
		void Create (
			MDLGPARENT dlgparent,						//! Parent
			DLGSTYLE dlgstyle = DLGSTYLE_Default	//! Style
			);

		//! Report that process is finished, with time displayed, and wait for OK.
		//! If the TEXTID__None is specified a generic 'time to process' message will be shown.
		void ReportDone (
			TEXTID textid = TEXTID__None
			);

		//! Report that process is finished, with message, and wait for OK
		void ReportDone (
			const MISTRING& msgstr
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_STATUS THISCLASS;
		class PRIVCONTEXT;
		MDLGPARENT m_dlgparent;
		DLGSTYLE m_dlgstyle;
		MGUI::CANVAS m_Canvas;
		MGUI::CTRL_LABEL m_Graphic;
		MGUI::CTRL_TEXT m_TextElapsed;
		MGUI::CTRL_SEPARATOR m_Separator;
		MGUI::CTRL_PUSHBUTTON *m_pBtnCancel;
		MGUI::CTRL_PUSHBUTTON *m_pBtnOK;
		MGUI::TIMER m_UpdateTimer;
		bool m_CancelPressed;
		bool m_NeedRecalcRows;
		bool m_NeedRecalcWidth;
		INT16 m_NumRows;
		INT16 m_Width;
		INT16 m_RowHeight;
		INT16 m_TextAscent;
		UINT32 m_LastTime;
		MGD::CONTEXT *m_pGC;

		void OnCanvasPaint (MGD::CONTEXT *gc);
		void OnTimerUpdate ();
		void Update (bool RedrawAll = false, MGD::CONTEXT *gc = 0);
		void WaitForOK ();

		// MISTATUSMANAGER/LOGGER overrides.
		virtual CONTEXT_PRIV* v_ContextPrivCreate (MISTATUSCONTEXT& Context);
		virtual bool v_IsCancelled ();
		virtual void v_ReportMessage (MISTATUSCONTEXT& context, const MISTRING& msgstr);
		virtual void v_ReportProgress (MISTATUSCONTEXT& context, double pctdone);
		virtual void v_ReportTimerValue (const MTIMER& timer);
		virtual void v_SetCancelEnabled (bool enabled);
		virtual void v_UpdateUI ();

		// DLGSHELL overrides.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCancel ();
		virtual void v_OnOpen ();

		// Unimplemented.
		DLG_STATUS (const DLG_STATUS&);
		DLG_STATUS& operator= (const DLG_STATUS&);
		#endif
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_STATUS::DLGSTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGSTATUS_H
