/**
 * \file mgui/extendedtooltip.h
 * \brief Definitions for MGUI::EXTENDED_TOOLTIP class
 *
 * \if NODOC
 * $Id: extendedtooltip.h_v 1.4 2011/09/01 19:31:21 mju Exp $
 *
 * $Log: extendedtooltip.h_v $
 * Revision 1.4  2011/09/01 19:31:21  mju
 * Fix dox hdr.
 *
 * Revision 1.3  2010/10/19 21:47:33  bcolombe
 * Add fontsize
 *
 * Revision 1.2  2010/10/14 23:02:35  bcolombe
 * *** empty log message ***
 *
 * Revision 1.1  2010/10/04 21:20:03  bcolombe
 * Initial revision
 *
 *
 * \endif
**/

#ifndef	INC_MGUI_EXTENDED_TOOLTIP_H
#define	INC_MGUI_EXTENDED_TOOLTIP_H

#ifndef INC_MGUI_TOOLTIP_H
#include <mgui/tooltip.h>
#endif

#ifndef INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

namespace MGUI {

class EXTENDED_TOOLTIP {
	public:

		//! Constructor.
		EXTENDED_TOOLTIP (
			MGUI::CTRL_TOOLTIP* tooltip,	//!< Tooltip to add text & extents image to
			MGUI::CTRL* parent				//!< Tooltip parent
			);

		//! Destructor.
		virtual ~EXTENDED_TOOLTIP ();

		//! Append extents text with image tooltip using Blue Marble as background
		void AddExtents (
			const DRECT2D& extents,
			const SPATREF::COORDREFSYS& crs
			);

		//! Append regular text
		void AddText (
			const MISTRING& string
			);

		//! Append text with label and value
		void AddText (
			const MISTRING& label,
			const MISTRING& value,
			bool bNewColumn = false
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MGUI::CTRL_TOOLTIP* m_tooltip;
		MGUI::CTRL* m_parent;
		MISTRINGLIST m_labels;
		MISTRINGLIST m_values;
		int m_fontsize;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLTIP_H
