/**
 * \file mgui/menuedit.h
 * \brief MGUI::MENU_EDIT class definitions
 *
 * \if NODOC
 * $Id: menuedit.h_v 1.4 2013/01/23 16:17:46 mju Exp $
 *
 * $Log: menuedit.h_v $
 * Revision 1.4  2013/01/23 16:17:46  mju
 * Add command_SelectAll.
 *
 * Revision 1.3  2011/09/01 19:37:00  mju
 * Fix dox hdr.
 *
 * Revision 1.2  2011/06/22 13:35:07  mju
 * Doc update.
 *
 * Revision 1.1  2007/06/14 22:15:42  dwilliss
 * Initial revision
 * \endif
 **/

#ifndef	INC_MGUI_MENUEDIT_H
#define	INC_MGUI_MENUEDIT_H

#ifndef	INC_MGUI_MENU_H
#include <mgui/menu.h>
#endif

#ifndef	INC_MI32_EDITMENU_H
#include <mi32/editmenu.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Menu supporting "edit" commands and automatic connection to "EDIT" controls.
//! When placed on a menu bar, any controls in the same shell that are derived
//! from CTRL_EDIT_BASE automatically register themselves and handle enabling
//! and disabling Cut and Copy based on whether or not there is a selection active.
class MENU_EDIT : public MENU {
	public:

		//! Command IDs.  Note: these can also be used IDs to SetEnabledID()
		enum COMMAND {
			COMMAND_Cut 				= 0x00010000,
			COMMAND_Copy 				= 0x00020000,
			COMMAND_Paste 				= 0x00040000,
			COMMAND_EraseSelection	= 0x00080000,
			COMMAND_Clear 				= COMMAND_EraseSelection,
			COMMAND_SelectAll			= 0x00100000,
			COMMAND_InsertFile		= 0x00200000,
			COMMAND_Find 				= 0x00400000,
			COMMAND_FindAgain			= 0x00800000,
			COMMAND_GoToLine			= 0x01000000,
			COMMAND_CharacterMap		= 0x02000000,
			COMMAND_Format				= 0x04000000,	//!< For WYSIWYG controls only
			COMMAND_ShowFormatCodes	= 0x08000000,	//!< For WYSIWYG controls only
			COMMAND_DEFAULT			= COMMAND_Cut | COMMAND_Copy | COMMAND_Paste | COMMAND_EraseSelection | COMMAND_SelectAll
			};

		//! Default constructor.
		MENU_EDIT ();

		//! Copy constructor.
		MENU_EDIT (const MENU_EDIT& rhs);

		//! Destructor.
		~MENU_EDIT ();

		//! Add one or more standard edit menu commands to the menu.
		void AddCommands (COMMAND commands = COMMAND_DEFAULT);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		EDITMENU m_EditMenuPriv;

		//! Assignment. (private and unimplemented)
		//MENU_EDIT& operator= (const MENU_EDIT& rhs);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MENU_EDIT::COMMAND)
#endif

//=====================================================================================================================
}	//! End namespace MGUI

#endif	// INC_MGUI_MENUEDIT_H
