/**
 * \file colorquantraster.h <mi32/colorquantraster.h>
 * \brief Color quantizer using RVC::RASTER source
 *
 * \if NODOC
 * $Id: colorquantraster.h_v 1.3 2007/01/19 16:09:24 mju Exp $
 *
 * $Log: colorquantraster.h_v $
 * Revision 1.3  2007/01/19 16:09:24  mju
 * Use color_array.
 *
 * Revision 1.2  2006/06/28 13:46:08  mju
 * No const on starTarget with objitem.
 *
 * Revision 1.1  2005/05/20 17:58:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COLORQUANTRASTER_H
#define	INC_MI32_COLORQUANTRASTER_H

#ifndef	INC_MI32_COLORQUANT_H
#include <mi32/colorquant.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif


class COLORQUANTIZER_RASTER : public COLORQUANTIZER {
	public:

		//! Default constructor.
		COLORQUANTIZER_RASTER (
			);

		//! Destructor.
		virtual ~COLORQUANTIZER_RASTER (
			);

		//! Get target raster.
		RVC::RASTER& GetTargetRaster (
			) { return (m_RasterTarget); }

		//! Set source from RVC::OBJITEM.
		ERRVALUE SetSource (
			const RVC::OBJITEM& ObjItem
			);

		//! Set source from RVC::RASTER.
		ERRVALUE SetSource (
			const RVC::RASTER& Raster
			);

		//! Set target from RVC::OBJITEM.
		ERRVALUE SetTarget (
			RVC::OBJITEM& ObjItem
			);

		//! Set target from RVC::RASTER.
		ERRVALUE SetTarget (
			const RVC::RASTER& Raster
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVC::RASTER m_RasterSource;
		RVC::NULLMASK m_MaskSource;
		RVC::RASTER m_RasterTarget;
		SIMPLE_ARRAY<UINT8> m_BufSource;
		SIMPLE_ARRAY<UINT8> m_BufSourceMask;
		SIMPLE_ARRAY<UINT8> m_BufTarget;

		ERRVALUE SetupSource ();

		// COLORQUANTIZER overrides.
		virtual ERRVALUE v_EndTargetRow (INT32 row);
		virtual INT32 v_GetNumColumns ();
		virtual INT32 v_GetNumRows ();
		virtual COLORSPACE v_GetSourceColorspace ();
		virtual ERRVALUE v_GetSourceRow (INT32 row, const void *& pImageRow, const UINT8 *& pMaskRow);
		virtual ERRVALUE v_GetTargetRow (INT32 row, UINT8 *& ptr);
		virtual ERRVALUE v_PutTargetPalette (COLOR_ARRAY& palette);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_COLORQUANTRASTER_H

