/**
 * \file filtbase.h <mi32/filtbase.h>
 * \brief FILTER_BASE class definition
 *
 * \if NODOC
 * $Id: filtbase.h_v 1.2 2003/09/15 13:49:56 fileserver!dwilliss Exp $
 *
 * $Log: filtbase.h_v $
 * Revision 1.2  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/01/16 15:40:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FILTBASE_H
#define	INC_MI32_FILTBASE_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


//===================================================================================================================
//! Base class for filters generally called by dialogs.
//! Provides management methods for derived classes to set message TextID and error values.
class FILTER_BASE {
	public:

		//! Clear current TEXTID and error value settings.
		void Clear (
			) { m_TextID = TEXTID__None; m_ErrValue = 0; }

		//! Get current error value.
		ERRVALUE GetErrValue (
			) const { return (m_ErrValue); }
			
		//! Get message based on the stored TEXTID and error value.
		//! If both error value and a TEXTID are set the error value will take precedence.
		MISTRING GetMessage (
			TEXTID TextID = TEXTID__None		//!< Text ID to use for message if none previously set
			) const;
			
		//! Get current TEXTID setting.
		TEXTID GetTextID (
			) const { return (m_TextID); }
			
	protected:
			
		//! Default constructor.
		FILTER_BASE (
			): m_TextID(TEXTID__None), m_ErrValue(0) { }

		//! Destructor.
		~FILTER_BASE () { }

		//! Set error value.
		void SetErrValue (
			ERRVALUE err
			) { m_ErrValue = err; }
			
		//! Set the current TEXTID for this filter
		void SetTextID (
			TEXTID TextID
			) { m_TextID = TextID; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TEXTID m_TextID;
		ERRVALUE m_ErrValue;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};
//===================================================================================================================


#endif	//!< INC_MI32_FILTBASE_H

