/**
 * \file getstrf.h <mi32/getstrf.h>
 * \brief GETSTRINGFILTER class definition
 *
 * \if NODOC
 * $Id: getstrf.h_v 1.3 2004/03/04 18:46:40 mju Exp $
 *
 * $Log: getstrf.h_v $
 * Revision 1.3  2004/03/04 18:46:40  mju
 * Add virtual dtor.
 *
 * Revision 1.2  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/01/16 16:06:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_GETSTRF_H
#define	INC_MI32_GETSTRF_H

#ifndef	INC_MI32_FILTBASE_H
#include <mi32/filtbase.h>
#endif

//-------------------------------------------------------------------------------------------------------------------
//!	Base class for string filter.
class GETSTRINGFILTER : public FILTER_BASE {
	public:

		//! Determine if string is valid.
		//! @return 'true' if valid, 'false' if not.
		bool IsValid (
			const MISTRING& string
			) { Clear(); return (v_IsValid(string)); }

		//! Modify string to make valid.
		//! @return 'true' if string modified, 'false' if not.
		bool MakeValid (
			MISTRING& string
			) { return (v_MakeValid(string)); }

	protected:

		virtual ~GETSTRINGFILTER () = 0;

	private:

		//! Called by IsValid() to determine if string is valid.
		//! @return 'true' if valid, 'false' if not.
		//! Default implementation always returns 'true'.
		virtual bool v_IsValid (
			const MISTRING& string
			);

		//! Modify string to make valid.
		//! @return 'true' if string modified, 'false' if not.
		//! If string is modified it must then be 'valid'.
		//! Default implementation leaves string unmodified and always returns 'false'.
		virtual bool v_MakeValid (
			MISTRING& string
			);
	};


#endif	//!< INC_MI32_GETSTRF_H
