/**
 * \file mi32/lidarclassifiermc.h
 * \brief LIDAR::CLASSIFIER_MULTISCALECURVATURE definitions
 *
 * \if NODOC
 * $Id: lidarclassifiermc.h_v 1.2 2012/09/21 15:12:57 mju Exp $
 *
 * $Log: lidarclassifiermc.h_v $
 * Revision 1.2  2012/09/21 15:12:57  mju
 * Add ability to save intermediate surfaces.
 *
 * Revision 1.1  2012/08/24 21:25:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARCLASSIFIERMC_H
#define	INC_MI32_LIDARCLASSIFIERMC_H

#ifndef	INC_MI32_LIDARCLASSIFIER_H
#include <mi32/lidarclassifier.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

namespace LIDAR {

//=====================================================================================================================

//! Multiscale Curvature Classifier to classify points as "Ground" or "Unclassified" (non-ground).
class CLASSIFIER_MULTISCALECURVATURE : public CLASSIFIER {
	public:

		//! Constructor.
		CLASSIFIER_MULTISCALECURVATURE ();

		//! Destructor.
		~CLASSIFIER_MULTISCALECURVATURE ();

		double GetCurvatureTolerance () const
			{ return (m_CurvatureTolerance); }

		double GetPointSpacing () const
			{ return (m_PointSpacing); }

		ERRVALUE Run (
			LIDAR::LAS::DATASET& dataset,
			const DRECT2D& extents
			);

		void SetCurvatureTolerance (
			double CurvatureTolerance
			) { m_CurvatureTolerance = CurvatureTolerance; }

		void SetPointSpacing (
			double PointSpacing
			) { m_PointSpacing = PointSpacing; }

		void SetSaveSurface (
			const FILEPATH& SaveFilePath
			) { m_SaveSurface = true; m_FilePathSaveSurface = SaveFilePath; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum OUTPUT {
			OUTPUT_Ground = 0,
			OUTPUT_Unclassified = 1
			};

		class ENGINE;

		double m_PointSpacing;
		double m_CurvatureTolerance;
		FILEPATH m_FilePathSaveSurface;
		bool m_SaveSurface;

		// Overrides.
		virtual double v_GetMargin () const;
		virtual UINT64 v_GetMaxPointsDesired () const;
		virtual void v_IniRead (INIHANDLE inihnd);
		virtual void v_IniWrite (INIHANDLE inihnd) const;
		virtual void v_JobGetValues (MIJOB& mijob) const;
		virtual ERRVALUE v_JobRead (const MIJOB& mijob);
		virtual void v_Reset ();
		#endif
	};

//=====================================================================================================================

}	// End namespace LIDAR

#endif	// INC_MI32_LIDARCLASSIFIERMC_H
