/**
 * \file mi32/strtodeg.h
 * \brief String / Degrees conversion functions
 *
 * \if NODOC
 * $Id: strtodeg.h_v 1.1 2011/11/03 13:34:44 mju Exp $
 *
 * $Log: strtodeg.h_v $
 * Revision 1.1  2011/11/03 13:34:44  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STRTODEG_H
#define	INC_MI32_STRTODEG_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


#ifdef GEOMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

//!\addtogroup latlonconv Latitude/Longitude Format Conversions
//!@{


enum DEGTOSTRFLAGS {
	DEGTOSTRFLAG_NoDir =				0x00000000,		//!< Don't include hemisphere (N,S,E,W) indicator
	DEGTOSTRFLAG_Longitude =		0x00000001,		//!< Format as longitude with E/W hemisphere indicator
	DEGTOSTRFLAG_Latitude =			0x00000002,		//!< Format as latitude with N/S hemisphere indicator
	DEGTOSTRFLAG_DirLast =			0x00000010,		//!< Put hemisphere indicator last
	DEGTOSTRFLAG_DecDegs =			0x00000020,		//!< Format as decimal degrees
	DEGTOSTRFLAG_DegDecMin =		0x00000040,		//!< Format as degrees and decimal minutes
	DEGTOSTRFLAG_DegMinSec =		0x00000080,		//!< Format as degrees, minutes and decimal seconds
	DEGTOSTRFLAG_Longitude360 =	0x00000100,		//!< Display in range of W360 to E360
	DEGTOSTRFLAG_AutoPrecision =	0x00000200,		//!< Automatically determine precision, not including seconds/minutes if unnecessary
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DEGTOSTRFLAGS)
#endif

//! Convert Latitude or Longitude value to char* string.
//! @return Pointer to string
//!	@see ConvLatLonToString, ConvertStringToDegs
LIBEXPORT char* ConvertDegsToString (
	char* string,								//!< String to hold result
	double value,								//!< Value to convert in degrees
	DEGTOSTRFLAGS flags						//!< Flags defining format
	);

//! Convert Latitude or Longitude value to MISTRING.
//! @return MISTRING containing converted value.
//!	@see ConvLatLonToString, ConvertStringToDegs
LIBEXPORT MISTRING ConvertDegsToString (
	double value,								//!< Value to convert in degrees
	DEGTOSTRFLAGS flags						//!< Flags defining format
	);

//! Convert Latitude or Longitude string to decimal degrees.
//!
//! @return true if successful, false if not
//!
//! This function automatically recognizes the following formats:
//!
//!		D ddd mm ss.sss			D indicates hemisphere (N, S, E or W)
//!		D ddd mm.mmmm				and can be anywhere in the string
//!		D ddd.ddddd
//!
//!    [-]ddd mm ss.sss       Sign (+ or -) is optional
//!    [-]ddd mm.mmmm         Negative values indicate West and South
//!    [-]ddd.ddddd           Do not type brackets []
//!
//!	@see ConvertStringToLatLon, ConvertDegsToString
LIBEXPORT bool ConvertStringToDegs (
	const char* string,
	double* value
	);

//!@}

#undef LIBEXPORT

#endif	// INC_MI32_STRTODEG_H
