/**
 * \file rvc/opcomponent.h
 * \brief RVC::OP::COMPONENT class definitions.
 *
 * \if NODOC
 * $Id: opcomponent.h_v 1.1 2009/06/25 20:50:49 mju Exp $
 *
 * $Log: opcomponent.h_v $
 * Revision 1.1  2009/06/25 20:50:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPCOMPONENT_H
#define	INC_RVC_OPCOMPONENT_H

#ifndef	INC_MI32_MISTRING_
#include <mi32/mistring.h>
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Encapsulate selection and valuation of component elements.
class COMPONENT {
	public:

	   //! Available selections of source elements.
		enum SELECTION {
			SELECTION_None =		0,
			SELECTION_All =		1,
			SELECTION_ByQuery =	2
			};

	   //! Available valuations of source elements.
		enum VALUATION {
			VALUATION_ByObject =	0,
			VALUATION_ByQuery =	1
			};

		//! Constructor.
		COMPONENT ();

		//! Copy constructor.
		COMPONENT (
			const COMPONENT& rhs
			);

		//! Destructor.
		~COMPONENT (
			);

		//! Assignment.
		COMPONENT& operator= (
			const COMPONENT& rhs
			);

		//! Get selection setting.
		SELECTION GetSelection (
			) const { return (m_Selection); }

		//! Get selection query string.
		const MISTRING& GetSelectionQuery (
			) const { return (m_SelectionQuery); }

		//! Get valuation setting.
		VALUATION GetValuation (
			) const { return (m_Valuation); }

		//! Get valuation query string.
		const MISTRING& GetValuationQuery (
			) const { return (m_ValuationQuery); }

		//! Set selection.
		void SetSelection (
			const SELECTION select
			) { m_Selection = select; }

		//! Set selection query string.
		void SetSelectionQuery (
			const MISTRING& query
			) { m_SelectionQuery = query; }

		//! Set valuation.
		void SetValuation (
			const VALUATION value
			) { m_Valuation = value; }

		//! Set valuation query string.
		void SetValuationQuery (
			const MISTRING& query
			) { m_ValuationQuery = query; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SELECTION m_Selection;
		VALUATION m_Valuation;
		MISTRING m_SelectionQuery;
		MISTRING m_ValuationQuery;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}  // End namespace OP
}  // End namespace RVC

#endif	// INC_RVC_OPCOMPONENT_H
