/**
 * \file imagefilterdivide.h <spatmod/imagefilterdivide.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_DIVIDE definition.
 *
 * \if NODOC
 * $Id: imagefilterdivide.h_v 1.2 2008/01/03 18:50:46 bcolombe Exp $
 *
 * $Log: imagefilterdivide.h_v $
 * Revision 1.2  2008/01/03 18:50:46  bcolombe
 * *** empty log message ***
 *
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERDIVIDE_H
#define	INC_SPATMOD_IMAGEFILTERDIVIDE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to divide image 'dividend' by image 'divisor'
//! Allows division of multiple components or combination of single and multi-component images.
//! Multi-component stages must have same number of components.
class FILTER_DIVIDE : public FILTER {
	public:

		//! Both stages must have same total dimensions.
		//! If no pixel properties are specified, the resulting sample data types will either be single or double precision float
		FILTER_DIVIDE (
			STAGE *pStageDividend,					//!< Pointer to first stage
			bool TakeOwnershipDividend,				//!< Filter takes ownership of previous stage and will delete when needed
			STAGE *pStageDivisor,					//!< Point to second stage
			bool TakeOwnershipDivisor,				//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL *pPixelProperties = 0		//!< Resulting pixel properties, 0 to automatically determine
			);

		// Destructor.
		virtual ~FILTER_DIVIDE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_DIVIDE (const FILTER_DIVIDE&);
		FILTER_DIVIDE& operator= (const FILTER_DIVIDE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERDIVIDE_H

