/**
 * \file spatmod/imagesourcetileset.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_TILESET definition.
 *
 * \if NODOC
 * $Id: imagesourcetileset.h_v 1.55 2012/07/10 14:56:42 bcolombe Exp $
 *
 * $Log: imagesourcetileset.h_v $
 * Revision 1.55  2012/07/10 14:56:42  bcolombe
 * Make tileset methods public
 *
 * Revision 1.54  2012/05/08 14:52:42  bcolombe
 * *** empty log message ***
 *
 * Revision 1.53  2012/05/04 21:41:54  bcolombe
 * Add CacheClear
 *
 * Revision 1.52  2012/04/24 17:52:19  bcolombe
 * Pass imgformat to checkcontenttype
 *
 * Revision 1.51  2012/04/18 16:06:03  bcolombe
 * Add TILETODOWNLOAD
 *
 * Revision 1.50  2012/04/09 14:40:09  mju
 * Don't include httpmsg hdr when should forward declare HTTPMESSAGE instead.
 *
 * Revision 1.49  2012/04/03 20:53:20  bcolombe
 * Changes for new terrain format
 *
 * Revision 1.48  2012/03/14 21:59:49  bcolombe
 * Changes for 24,32 bit terrain
 *
 * Revision 1.47  2012/03/06 16:53:13  bcolombe
 * *** empty log message ***
 *
 * Revision 1.46  2012/03/06 01:26:26  bcolombe
 * Changes for using tilemap
 *
 * Revision 1.45  2012/02/27 17:21:37  bcolombe
 * Make PullTiles public method
 *
 * Revision 1.44  2012/02/20 17:56:44  bcolombe
 * Support new png16 format
 *
 * Revision 1.43  2012/02/09 17:40:31  bcolombe
 * Add bFoundTiles to PullTiles
 *
 * Revision 1.42  2012/01/30 20:58:42  bcolombe
 * Include objitem header
 *
 * Revision 1.41  2012/01/27 21:52:52  bcolombe
 * Add PullTiles
 *
 * Revision 1.40  2012/01/27 19:16:35  bcolombe
 * Add v_ComputePixelRange
 * Change terrain tileset to be grayscale no alpha
 *
 * Revision 1.39  2011/09/09 18:25:02  bcolombe
 * AddTileLabels: add zoom level as param
 *
 * Revision 1.38  2011/05/06 22:07:57  bcolombe
 * Add CheckContentType
 *
 * Revision 1.37  2011/05/04 23:56:15  bcolombe
 * AddSectionToDownload
 *
 * Revision 1.36  2011/04/05 15:27:40  mju
 * Add v_queryIterator.
 *
 * Revision 1.35  2011/04/04 16:05:17  bcolombe
 * Add v_ProcessBottomTile for bing
 *
 * Revision 1.34  2011/03/31 15:06:20  mju
 * Major update to eliminate code duplication and simplify job management.
 * Eliminate unused members.
 * Use logging from image::base.
 *
 * Revision 1.33  2011/02/14 22:03:16  bcolombe
 * Add GetTileIdx, FindTileFile
 *
 * Revision 1.32  2011/02/07 23:24:12  bcolombe
 * Add v_GetDatatip
 * Add v_GetDatatipObjectNames
 *
 * Revision 1.31  2011/02/04 21:58:20  bcolombe
 * Add v_GetDataTip.
 *
 * Revision 1.30  2011/02/04 18:26:52  bcolombe
 * Construct from objitem
 * Add v_GetCacheName
 *
 * Revision 1.29  2011/02/04 14:15:08  mju
 * Forward declare gre::layerdc.
 * NOTE: spatmod headers should not be using gre classes at all.
 *
 * Revision 1.28  2011/02/03 21:41:27  mju
 * Add v_getGeoreference.
 *
 * Revision 1.19  2010/04/29 21:27:45  bcolombe
 * Added LastZipFile and LastZipFilePath members
 *
 * Revision 1.16  2010/02/22 20:28:10  bcolombe
 * Change datatip to take current view cellsize
 *
 * Revision 1.15  2010/02/17 23:31:57  bcolombe
 * Added SetMaxLevelToUse
 *
 * Revision 1.11  2009/11/30 20:29:02  bcolombe
 * Added CheckCache
 *
 * Revision 1.10  2009/11/11 22:47:10  bcolombe
 * Added GetMaxScaleVisible
 *
 * Revision 1.9  2009/11/11 21:26:44  bcolombe
 * Added GetMinLevel, GetMaxLevel
 *
 * Revision 1.8  2009/11/05 23:44:41  bcolombe
 * Added GetMinLevelCellSize
 *
 * Revision 1.7  2009/10/22 19:22:11  bcolombe
 * Now uses GetHREFStr to get path
 *
 * Revision 1.6  2009/09/28 20:07:55  bcolombe
 * Added transparent colors
 *
 * Revision 1.5  2009/09/23 22:51:30  bcolombe
 * Add drawing of tile border and labels
 *
 * Revision 1.3  2009/08/17 16:23:30  bcolombe
 * Added GetMaxLevelCellSize
 *
 * Revision 1.1  2009/07/24 23:18:42  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCETILESET_H
#define	INC_SPATMOD_IMAGESOURCETILESET_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

#ifndef INC_SPATMOD_IMAGETILESET_H
#include <spatmod/imagetileset.h>
#endif

#ifndef INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef INC_MI32_DATETIME_H
#include <mi32/datetime.h>
#endif

#ifndef INC_MI32_CRITSEC_H
#include <mi32/critsec.h>
#endif

#ifndef INC_MI32_ZIPFILE_H
#include <mi32/zipfile.h>
#endif

#ifndef INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class HTTPMESSAGE;
namespace GRE {
	class LAYERDC;
	}
#endif


namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

class SOURCE_TILESET : public SOURCE {
	public:

		//! Clear tiles cache
		static void CacheClear ();

		//! Datatip flags
		enum DATATIPFLAG {
			DATATIPFLAG_None =			0x0000,
			DATATIPFLAG_TileInfo =		0x0001,		//!< Include tile info in datatip (level, row, column)
			DATATIPFLAG_ImageValue =	0x0002,		//!< Include pixel value in datatip
			DATATIPFLAG_Date =			0x0008,		//!< Include date info for tile in datatip
			};

		//! Constructor.
		SOURCE_TILESET (
			const RVC::OBJITEM& objitem
			);

		//! Constructor.
		SOURCE_TILESET (
			);

		// Destructor.
		virtual ~SOURCE_TILESET ();

		//! Draw tile labels overlay
		ERRVALUE AddTileLabels (
			INT32 ZoomLevelToUse,		//!< Current zoom level
			const LRECT2D& ObjRect,		//!< Device rect used
			const GRE::LAYERDC& layerdc	//!< Layer dc to draw on
			);
		//! Find tile (local or cached, will download if necessary)
		ERRVALUE FindTileFile (
			INT32 ZoomLevelToUse,		//!< Current zoom level
			const LPOINT2D& TileIdx,	//!< Tile
			FILEPATH& PathTileFile		//!< Path to tile returned
			);
		//! Get datatip string
		ERRVALUE GetDatatip (
			const DPOINT2D& point,		//!< Object coord point
			MISTRING& string,			//!< Datatip string returned
			INT32 ZoomLevelToUse,		//!< Current zoom level
			DATATIPFLAG flags			//!< Datatip info to include
			) { return (v_GetDatatip(point, string, ZoomLevelToUse, flags)); }
		//! Get datatip with just tile info
		ERRVALUE GetDatatipObjectNames (
			const DPOINT2D& point,		//!< Object coord point
			MISTRING& string,			//!< Datatip string returned
			INT32 ZoomLevelToUse,		//!< Current zoom level
			int indent = 0				//!< Indent amount
			) { return (v_GetDatatipObjectNames(point, string, ZoomLevelToUse, indent)); }
		//! Get datestring set for tileset
		const MISTRING& GetDateString () const
			{ return (m_DateString); }
		//! Get maximum zoom level of tileset
		INT32 GetMaxLevel () const
			{ return (m_TileSet.GetMaxLevel()); }
		//! Get minimum zoom level of tileset
		INT32 GetMinLevel () const
			{ return (m_TileSet.GetMinLevel()); }
		//! Get cell size for maximum zoom level of tileset
		double GetMaxLevelCellSize () const;
		//! Get cell size for minimum zoom level of tileset
		double GetMinLevelCellSize () const;
		//! Get maximum visible scale given pixel size of monitor
		double GetMaxScaleVisible (
			double PixelSizeMM
			) const;
		//! Get tile given object coord point
		ERRVALUE GetTileIdx (
			LPOINT2D& TileIdx,			//!< Tile returned
			const DPOINT2D& point,		//!< Object coord point
			INT32 ZoomLevel				//!< Current zoom level
			);
		//! Get tile and pixel in tile given object coord point
		ERRVALUE GetTileIdx (
			LPOINT2D& TileIdx,			//!< Tile returned
			LPOINT2D& PixelInTile,		//!< Pixel in tile returned
			const DPOINT2D& point,		//!< Object coord point
			INT32 ZoomLevel				//!< Current zoom level
			);
		//! Get TILESET class for source
		SPATMOD::IMAGE::TILESET& GetTileSet ()
			{ return (m_TileSet); }
		//! Determine if tileset has tile map
		bool HasTileMap ();
		//! Find tile in tile map read buffer
		FORMAT::SPEC FindTileInTileMap (
			const SPATMOD::IMAGE::BUFFER& Buffer,
			INT32 ZoomLevel,
			const LPOINT2D& TopLeftTileIdx,
			const LPOINT2D& TileIdx
			);
		//! Read tile map into buffer
		ERRVALUE ReadTileMap (
			SPATMOD::IMAGE::BUFFER& Buffer,
			INT32 level,
			const LPOINT2D& TopLeftTileIdx,
			const LPOINT2D& BottomRightTileIdx,
			bool bIsMaxRangeTileMap = false
			);
		//! Override StagePtr's georeference with computed tileset georeference for given zoom factor
		ERRVALUE MakeGeoreference (
			STAGE_AUTOPTR& StagePtr,
			double ZoomUsed
			);
		//! Set date string to use for tileset
		//! If url string for tiles includes date entry, then tileset layer gives option to view specific dates
		void SetDateString (const MISTRING& DateString)
			{ m_DateString = DateString; }
		//! Set maximum level to use for tileset
		ERRVALUE SetMaxLevelToUse (INT32 MaxLevel);
		//! Option to turn off use of download threads
		void SetUseMultithreaded (bool bUseMultithreaded) {m_bUseMultithreaded = bUseMultithreaded;}
		
	protected:
		// SOURCE_TILESET - protected (used by derived SOURCE_BINGMAPS)

		//! Compute tile and pixel in tile position given starting row and column of section
		void ComputePosition (INT32 TileSize, INT32 RowsToAdd, INT32 ColsToAdd, const LPOINT2D& BaseTopLeftTileIdx, LPOINT2D& TileIdx, LPOINT2D& PixelInTile);
		//! Find and construct source for tile
		ERRVALUE FindSource (const MISTRING& RemotePathString, INT32 TileMapLevel, INT32 ZoomLevelToUse, const LPOINT2D& TileIdx, SPATMOD::IMAGE::STAGE*& source, bool bDownloadIfNeeded = false, MISTRING* PathString = 0, bool bReturnOnlyString = false);
		//! Find and construct source for tile
		ERRVALUE FindSource (INT32 ZoomLevelToUse, const LPOINT2D& TileIdx, SPATMOD::IMAGE::STAGE*& source, bool bDownloadIfNeeded = false, MISTRING* PathString = 0, bool bReturnOnlyString = false);
		//! Do tileset type specific initializtion
		ERRVALUE FinishInitialize ();
		//! Set if tile map should be used (false for BING)
		void SetUseTileMap (bool bUseTileMap) {m_bUseTilemap = bUseTileMap;}
		//! Datatip override
		virtual ERRVALUE v_GetDatatip (const DPOINT2D& point, MISTRING& string, INT32 ZoomLevelToUse, DATATIPFLAG flags);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// SOURCE_TILESET private methods

		//! Check if extension of downloaded tile matches reply content type
		//! for bing - can request and download a jpeg tile, even if content was actually png, so need to change extension
		static void CheckContentType (SPATMOD::IMAGE::FORMAT& ImgFormat, FILEPATH& PathTileFile, const HTTPMESSAGE& reply, const SPATMOD::IMAGE::TILESET& TileSet);
		//! Add current section tiles to download thread queue
		ERRVALUE AddSectionToDownload(const SPATMOD::IMAGE::SECTION& section);
		//! Check if cache is full
		void CheckCache ();
		//! Check if need to download tile (looks in local cache and tile map)
		ERRVALUE CheckDownloadTile (const LPOINT3D& TileIdx);
		//! Filter delegate to convert pixel properties of tile
		bool FilterDelegate (
			const SPATMOD::IMAGE::SAMPLEITERATOR *ittgt,
			const SPATMOD::IMAGE::SAMPLEITERATOR *itsrc,
			bool valid
			);
		//! Get current cache location
		const FILEPATH& GetCacheBasePath () const
			{ return (m_TempCacheBasePath); }
		//! Generate temp cache name for tile
		MISTRING GetCacheFileName (const LPOINT2D& TileIdx, INT32 ZoomLevel, INT32 TileMapLevel = -1);
		//! Determine zoom level to use from section zoom
		INT32 GetZoomLevel (INT32 MaxLevel, double zoom) const;
		//! Download thread finish delegate
		void OnJobDone (MITHREAD::ID id);
		//! Add job to download thread queue
		void QueueNextTileJob (int ServerPos);

		//! Pull tiles into buffer
		ERRVALUE PullTiles (
			const SECTION& SubSection,						//!< Section to pull
			BUFFER& buffer,									//!< Buffer to pulll data into
			const SPATMOD::IMAGE::PIXEL& PixelProperties,	//!< Pixel properties of data
			bool &bFoundTiles								//!< Were any tiles actually found?
			);
		//! Pull tiles into buffer
		ERRVALUE PullTiles (
			const MISTRING& RemotePathString,				//!< Specific url string to use
			INT32 TileMapLevel,								//!< Tile map level to use
			INT32 ZoomLevelToUse,							//!< Current zoom level
			const SECTION& SubSection,						//!< Section to pull
			BUFFER& buffer,									//!< Buffer to pulll data into
			const SPATMOD::IMAGE::PIXEL& PixelProperties,	//!< Pixel properties of data
			bool &bFoundTiles								//!< Were any tiles actually found?
			);
		//! Get pixel value from pixel range map read buffer
		void GetMinimumPixelInMap (
			const SPATMOD::IMAGE::BUFFER& Buffer,
			INT32 ZoomLevel,
			const LPOINT2D& TopLeftTileIdx,
			const LPOINT2D& TileIdx,
			double& MinValue
			);
		//! Get pixel value from pixel range map read buffer
		void GetMaximumPixelInMap (
			const SPATMOD::IMAGE::BUFFER& Buffer,
			INT32 ZoomLevel,
			const LPOINT2D& TopLeftTileIdx,
			const LPOINT2D& TileIdx,
			double& MaxValue
			);
		// Read pixel range for tile
		ERRVALUE ReadPixelRange (
			INT32 level, 
			const LPOINT2D& TileIdx,
			double& MinValue,
			double& MaxValue
			);

		// SOURCE_TILESET overridables.
		virtual void v_GetCacheName (MISTRING& CacheName);
		virtual ERRVALUE v_GetDatatipObjectNames (const DPOINT2D& point, MISTRING& string, INT32 ZoomLevelToUse, int indent);
		virtual ERRVALUE v_ProcessBottomTile (SPATMOD::IMAGE::STAGE*& source, INT32 ZoomLevelToUse, const LPOINT2D& TileIdx);

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ERRVALUE v_GetGeoreference (GEOREFERENCE& georeference, double zoom) const;
		virtual const char* v_GetTypeName () const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;

		// STAGE overrides.
		virtual ERRVALUE v_ComputePixelRange (
			const SECTION& section,						//!< Section, including desired zoom value if allowed
			PIXEL& PixelValueMin,						//!< Minimum pixel value returned
			PIXEL& PixelValueMax,						//!< Maximum pixel value returned
			RANGETYPE accuracy							//!< Desired range accuracy
			);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		SOURCE_TILESET (const SOURCE_TILESET&);
		SOURCE_TILESET& operator= (const SOURCE_TILESET&);

		// SOURCE_TILESET private members
		class TILEJOB;
		class TILETODOWNLOAD;

		bool m_bUseTilemap;
		INT32 m_NumTiles;
		MISTRING m_DateString;
		MISTRING m_TempCacheName;
		FILEPATH m_TempCacheBasePath;
		FILEPATH m_LastZipFilePath;
		ZIPFILE m_LastZipFile;
		RVC::OBJITEM m_objitem;
		SPATMOD::IMAGE::TILESET m_TileSet;
		SPATMOD::IMAGE::TILESET m_TileMapTileset;
		SPATMOD::IMAGE::TILESET::IMGFORMAT m_ImgFormat;
		SPATMOD::IMAGE::PIXEL::TYPE m_SrcPixelType;
		MILIST<TILEJOB*> m_TileJobPtrs;
		MILIST<TILETODOWNLOAD> m_TilesToDownload;
		MITHREAD::QUEUE m_Queue;
		MITHREAD::POOL* m_Pool;
		MITHREAD::CRITICALSECTION m_TileListSync;
		bool m_bUseMultithreaded;

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SOURCE_TILESET::DATATIPFLAG)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEGOOGLEMAPS_H

